@@ -8,6 +8,8 @@ import rasterio.features
 import warnings
 import numpy as np
 import json as js
+import tkinter as tk
+from tkinter import messagebox
 
 warnings.filterwarnings("ignore", category=RuntimeWarning, module="shapely.set_operations")
 
@@ -58,10 +60,14 @@ def find_contour_centroid(contour):
     centroid_x = int(M["m10"] / M["m00"])
     centroid_y = int(M["m01"] / M["m00"])
 
-    return (centroid_x, centroid_y)
+    return centroid_x, centroid_y
 
 
-def convert_contours_to_coco(contours, height, width, img_name, id, coco_data):
+def convert_contours_to_coco(outer_contours, inner_contours, height, width, img_name, start_id, coco_data):
+    new_id = start_id  # Starting ID for annotations
+    outer_category_id = 1  # ID pro štítek "spheroids"
+    inner_category_id = 2  # ID pro štítek "holes"
+
     # Add image information
     image_info = {
         "id": len(coco_data["images"]) + 1,
@@ -76,35 +82,42 @@ def convert_contours_to_coco(contours, height, width, img_name, id, coco_data):
 
     coco_data["images"].append(image_info)
 
-    new_id = id  # Starting ID for annotations
+    def process_contours(contours, category_id):
+        nonlocal new_id
+        for contour in contours:
+            # Ensure each contour has enough points
+            if len(contour) < 3:
+                continue  # Skip contours with less than 3 points
 
-    for contour in contours:
-        if len(contour) % 2 == 1:
-            contour = contour[1:]
+            segmentation = contour.flatten().tolist()
 
-        segmentation = contour.flatten().tolist()
+            annotation_data = {
+                "id": new_id,
+                "image_id": image_info["id"],
+                "category_id": category_id,
+                "segmentation": [segmentation],
+                "area": float(cv.contourArea(np.array(contour))),
+                "bbox": cv.boundingRect(np.array(contour))[:4],
+                "iscrowd": 0
+            }
 
-        annotation_data = {
-            "id": new_id,
-            "image_id": image_info["id"],
-            "category_id": 1,
-            "segmentation": [segmentation],
-            "area": float(cv.contourArea(np.array(contour))),
-            "bbox": cv.boundingRect(np.array(contour))[:4],
-            "iscrowd": 0
-        }
+            coco_data["annotations"].append(annotation_data)
+            new_id += 1
 
-        coco_data["annotations"].append(annotation_data)
+    # Zpracování vnějších kontur
+    process_contours(outer_contours, outer_category_id)
 
-        new_id += 1
+    # Zpracování vnitřních kontur, pokud jsou k dispozici
+    if inner_contours:
+        process_contours(inner_contours, inner_category_id)
 
     return coco_data
 
 
-def Erosion(img, dilation_size=3, dilation_shape=cv.MORPH_ELLIPSE):
-    element = cv.getStructuringElement(dilation_shape, (2 * dilation_size + 1, 2 * dilation_size + 1),
-                                       (dilation_size, dilation_size))
-    img_final = cv.erode(img, element)
+def Erosion(img, erosion_size=3, iterations=1, erosion_shape=cv.MORPH_ELLIPSE):
+    element = cv.getStructuringElement(erosion_shape, (2 * erosion_size + 1, 2 * erosion_size + 1),
+                                       (erosion_size, erosion_size))
+    img_final = cv.erode(img, element, iterations=iterations)
 
     return img_final
 
@@ -118,10 +131,10 @@ def find_bounds(contour):
     return lower_bound, upper_bound, left_bound, right_bound
 
 
-def Dilation(img, dilation_size=3, dilation_shape=cv.MORPH_ELLIPSE):
+def Dilation(img, dilation_size=3, iterations=1, dilation_shape=cv.MORPH_ELLIPSE):
     element = cv.getStructuringElement(dilation_shape, (2 * dilation_size + 1, 2 * dilation_size + 1),
                                        (dilation_size, dilation_size))
-    img_final = cv.dilate(img, element)
+    img_final = cv.dilate(img, element, iterations=iterations)
 
     return img_final
 
@@ -242,13 +255,14 @@ def save_polygons(polygons_ref, polygons_pred, title, name, projekt, algorithm,
         plt.savefig(f"Results/{projekt}/IoU output/plots/{algorithm}_{name.replace('bmp', 'png')}")
 
     plt.close()
-def load_annotations(adresa_datasetu):
-    with open(adresa_datasetu) as f:
+
+
+def load_annotations(annotations_address, images_address):
+    with open(annotations_address) as f:
         data = js.load(f)
 
-    # Slovník pro uchování kontur a masek pro každý obrázek
-    contours_per_image = {}
     masks_per_image = {}
+    annotations_data = []
 
     for annotation in data["annotations"]:
         img_id = annotation["image_id"]
@@ -259,22 +273,23 @@ def load_annotations(adresa_datasetu):
             img_width = img_info["width"]
             img_height = img_info["height"]
 
+            img_path = os.path.join(images_address, img_name)
+            img = cv.imread(img_path)
+
             # Inicializace masky pro každý obrázek
             if img_name not in masks_per_image:
                 masks_per_image[img_name] = np.zeros((img_height, img_width), dtype=np.uint8)
-                contours_per_image[img_name] = []
 
             # Pro každý segment vytvořte kontury
             for segmentation in annotation["segmentation"]:
-                points = np.array([[segmentation[i], segmentation[i + 1]] for i in range(0, len(segmentation), 2)], dtype=np.int32)
+                points = np.array([[segmentation[i], segmentation[i + 1]] for i in range(0, len(segmentation), 2)],
+                                  dtype=np.int32)
                 points = points.reshape((-1, 1, 2))
-                contours_per_image[img_name].append(points)
                 cv.fillPoly(masks_per_image[img_name], [points], 255)
 
+            annotations_data.append((masks_per_image[img_name], img, img_name))
 
-    return list(contours_per_image.values()), list(masks_per_image.values()), list(contours_per_image.keys())
-
-
+    return annotations_data
 
 
 def checkPolygon(geom):
@@ -294,6 +309,14 @@ def IoU(projekt, algorithm, mask_ref, mask_pred, name, plot=False, save=True, lo
     union = np.logical_or(mask_ref, mask_pred)
     IoU = np.sum(intersection) / np.sum(union)
 
+    true_positive = np.sum(np.logical_and(mask_ref, mask_pred))
+    false_negative = np.sum(np.logical_and(mask_ref, np.logical_not(mask_pred)))
+    false_positive = np.sum(np.logical_and(np.logical_not(mask_ref), mask_pred))
+
+    # Výpočet True Positive Rate (TPR) a Positive Predictive Value (PPV)
+    tpr = true_positive / (true_positive + false_negative)
+    ppv = true_positive / (true_positive + false_positive)
+
     # Získání kontur z masek
     contours_ref, _ = cv.findContours(mask_ref.astype(np.uint8), cv.RETR_EXTERNAL, cv.CHAIN_APPROX_SIMPLE)
     contours_pred, _ = cv.findContours(mask_pred.astype(np.uint8), cv.RETR_EXTERNAL, cv.CHAIN_APPROX_SIMPLE)
@@ -312,13 +335,12 @@ def IoU(projekt, algorithm, mask_ref, mask_pred, name, plot=False, save=True, lo
     if save:
         with lock:
             if address:
-                save_polygons(polygon_ref, polygon_pred, "IoU: " + str(IoU * 100) + "%", name, projekt, algorithm, address=address)
+                save_polygons(polygon_ref, polygon_pred, "IoU: " + str(IoU * 100) + "%", name, projekt, algorithm,
+                              address=address)
             else:
                 save_polygons(polygon_ref, polygon_pred, "IoU: " + str(IoU * 100) + "%", name, projekt, algorithm)
 
-    return IoU
-
-
+    return IoU, tpr, ppv
 
 
 def f1_score(polygon1, polygon2):
@@ -503,12 +525,54 @@ def compute_optimal_radii(img_color, k):
 
     return spatial_radius, color_radius
 
-def filter_contours_by_min_area(contours, min_area):
-    filtered_contours = []
-    for contour in contours:
-        area = cv.contourArea(contour)
-        if area >= min_area:
-            filtered_contours.append(contour)
-    return filtered_contours
+def load_masks_from_images(mask_address, images_address):
+    mask_files = [f for f in os.listdir(mask_address) if f.endswith(('.png', '.jpg', '.jpeg', '.bmp', '.tiff'))]
+    data = []
 
+    for mask_file in mask_files:
+        mask_path = os.path.join(mask_address, mask_file)
+        img_path = os.path.join(images_address, mask_file)
 
+        if not os.path.exists(img_path):
+            messagebox.showerror("Error", f"Image not found for mask '{mask_file}'")
+            continue
+
+        mask = cv.imread(mask_path, cv.IMREAD_GRAYSCALE)
+        img = cv.imread(img_path)
+
+        if mask is None:
+            messagebox.showerror("Error", f"Failed to load mask {mask_file}.")
+            continue
+
+        # Check if the mask pixel values are only 0 and 255
+        unique_values = np.unique(mask)
+        if not np.array_equal(unique_values, [0, 255]) and len(unique_values) != 2:
+            messagebox.showerror("Error", f"File {mask_file} does not contain a binary mask. It contains unique values: {unique_values}")
+            continue
+
+        data.append((mask, img, mask_file))
+
+    return data
+
+def load_masks(mask_address):
+    mask_files = [f for f in os.listdir(mask_address) if f.endswith(('.png', '.jpg', '.jpeg', '.bmp', '.tiff'))]
+    data = []
+
+    for mask_file in mask_files:
+        mask_path = os.path.join(mask_address, mask_file)
+
+        mask = cv.imread(mask_path, cv.IMREAD_GRAYSCALE)
+
+        if mask is None:
+            messagebox.showerror("Error", f"Failed to load mask {mask_file}.")
+            continue
+
+        # Check if the mask pixel values are only 0 and 255
+        unique_values = np.unique(mask)
+        if not np.array_equal(unique_values, [0, 255]) and len(unique_values) != 2:
+            messagebox.showerror("Error", f"File {mask_file} does not contain a binary mask. It contains unique values: {unique_values}")
+            continue
+
+        data.append((mask, mask_file))
+
+    return data
